#include <Keypad.h>
#include <Wire.h>

int pinLED = 8;
int pinBuzzer = 9;
bool isBuzzerActive = false;
bool isPinEntered = false;
String enteredPin = "";
int remainingAttempts = 3;

#define I2C_SLAVE_ADDRESS 0x09
#define BAUDRATE 115200

static char i2c_byte = 0;
static char previousMotionState = 'n';

const byte ROWS = 4;
const byte COLS = 4;

char hexaKeys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

byte rowPins[ROWS] = {12, 11, 10, 6};
byte colPins[COLS] = {5, 4, 3, 2};

Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);

void receiveEvent(int numBytes) {
  i2c_byte = Wire.read();
}

void activateBuzzer() {
  isBuzzerActive = true;
  digitalWrite(pinLED, HIGH);
  tone(pinBuzzer, 500);
  Wire.beginTransmission(I2C_SLAVE_ADDRESS);
  Wire.write('y');
  Wire.endTransmission();
}

void deactivateBuzzer() {
  isBuzzerActive = false;
  digitalWrite(pinLED, LOW);
  noTone(pinBuzzer);
  Wire.beginTransmission(I2C_SLAVE_ADDRESS);
  Wire.write('n');
  Wire.endTransmission();
}

void checkPin() {
  if (enteredPin.equals("*123#")) {
    isPinEntered = true;
    deactivateBuzzer();
    Wire.beginTransmission(I2C_SLAVE_ADDRESS);
    Wire.write('d');
    Wire.endTransmission();
  } else {
    remainingAttempts--;
    if (remainingAttempts > 0) {
      Wire.beginTransmission(I2C_SLAVE_ADDRESS);
      Wire.write('i');
      Wire.write(remainingAttempts);
      Wire.endTransmission();
      //enteredPin = ""; // Reset entered pin
    } else {
      activateBuzzer();
      Wire.beginTransmission(I2C_SLAVE_ADDRESS);
      Wire.write('m');
      Wire.endTransmission();
    }
  }
  enteredPin = ""; // Reset entered pin

}

void setup() {
  pinMode(pinBuzzer, OUTPUT);
  pinMode(pinLED, OUTPUT);
  Serial.begin(BAUDRATE);
  //deactivateBuzzer();
  Wire.begin(I2C_SLAVE_ADDRESS);
  Wire.onReceive(receiveEvent);
  delay(100);
}

void loop() {
  if (i2c_byte != 0) {
    if (i2c_byte == 'n') {
      Serial.println("No motion detected");
      if (previousMotionState == 'y' && isPinEntered && !isBuzzerActive) {
        activateBuzzer();
      }
      previousMotionState = 'n';
    } else if (i2c_byte == 'y') {
      Serial.println("Motion detected");
      previousMotionState = 'y';
      if (!isPinEntered && !isBuzzerActive) {
        activateBuzzer();
      } else if (isPinEntered && isBuzzerActive) {
        deactivateBuzzer();
      }
    }
    i2c_byte = 0;
  }
  
  char customKey = customKeypad.getKey();
  
  if (customKey) {
    Serial.println(customKey);
    enteredPin += customKey;
    
    if (enteredPin.length() >= 5) {
      checkPin();
    }
  }
} 